// -*- C++ -*-
/*!
 * @file  CameraImageViewer.h
 * @brief ImageView
 * @date  $Date$
 *
 * @author メディアドライブ
 * rtc-dev@mediadrive.jp
 *
 * BSD 3-clause "New" or "Revised" License
 *
 * $Id$
 */

#ifndef CAMERAIMAGEVIEWER_H
#define CAMERAIMAGEVIEWER_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

using namespace RTC;

/*!
 * @class CameraImageViewer
 * @brief ImageView
 *
 * Cameraコンポーネントで取得した画像を表示するコンポーネントです。
 * ４台まで同時に表示します。
 *
 */
class CameraImageViewer
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  CameraImageViewer(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~CameraImageViewer();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 表示画像サイズ高
   * - Name:  image_height
   * - DefaultValue: 240
   * - Unit: pixel
   */
  int m_image_height;
  /*!
   * 表示画像サイズ幅
   * - Name:  image_width
   * - DefaultValue: 320
   * - Unit: pixel
   */
  int m_image_width;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::CameraImage m_inCam1;
  /*!
   * カメラ入力（1号機）
   * - Type: RTC::CameraImage
   */
  InPort<RTC::CameraImage> m_inCam1In;
  RTC::CameraImage m_inCam2;
  /*!
   * カメラ入力（2号機）
   * - Type: RTC::CameraImage
   */
  InPort<RTC::CameraImage> m_inCam2In;
  RTC::CameraImage m_inCam3;
  /*!
   * カメラ入力（3号機）
   * - Type: RTC::CameraImage
   */
  InPort<RTC::CameraImage> m_inCam3In;
  RTC::CameraImage m_inCam4;
  /*!
   * カメラ入力（4号機）
   * - Type: RTC::CameraImage
   */
  InPort<RTC::CameraImage> m_inCam4In;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::TimedLong m_select;
  /*!
   * カメラ指示
   * - Type: RTC::TimedLong
   */
  OutPort<RTC::TimedLong> m_selectOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>
  //
 private:
     int dummy;
//     IplImage* m_orig_img;
//     IplImage* small_image;
//     IplImage* scale_image;


     int m_nOldHeight;     // CFG更新チェック用
     int m_nOldWidth;

     int m_in_height;      // 入力イメージのHeight
     int m_in_width;       // 入力イメージのWidth

     int ImageProgress(RTC::CameraImage& m_in, std::string strGoron);
     bool isCFGChanged();
//
 public:
     friend void onMouse1(int nEvent, int x, int y, int nFlags, void* param);
     friend void onMouse2(int nEvent, int x, int y, int nFlags, void* param);
     friend void onMouse3(int nEvent, int x, int y, int nFlags, void* param);
     friend void onMouse4(int nEvent, int x, int y, int nFlags, void* param);

};


extern "C"
{
  DLL_EXPORT void CameraImageViewerInit(RTC::Manager* manager);
};

#endif // CAMERAIMAGEVIEWER_H
