// -*- C++ -*-
/*!
 * @file  CameraImageViewer.cpp
 * @brief ImageView
 * @date $Date$
 *
 * @author メディアドライブ
 * rtc-dev@mediadrive.jp
 *
 * BSD 3-clause "New" or "Revised" License
 *
 * $Id$
 */

#include "CameraImageViewer.h"
#include "ImageDataConvert.h"

// Module specification
// <rtc-template block="module_spec">
static const char* cameraimageviewer_spec[] =
  {
    "implementation_id", "CameraImageViewer",
    "type_name",         "CameraImageViewer",
    "description",       "ImageView",
    "version",           "1.0.0",
    "vendor",            "MEDIA DRIVE",
    "category",          "ImageProcessing",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.image_height", "240",
    "conf.default.image_width", "320",

    // Widget
    "conf.__widget__.image_height", "text",
    "conf.__widget__.image_width", "text",
    // Constraints

    "conf.__type__.image_height", "int",
    "conf.__type__.image_width", "int",

    ""
  };
// </rtc-template>


int selectedNum;
bool outputfull;
bool setfull;

#define SCALE 2.0
#define X_MARGIN 40
#define Y_MARGIN 40

void onMouse1(int nEvent, int x, int y, int nFlags, void *param)
{
    if(nEvent == CV_EVENT_LBUTTONDOWN)
    {
    	selectedNum = 1;
    	outputfull = true;
    	setfull = false;
    }
}

void onMouse2(int nEvent, int x, int y, int nFlags, void *param)
{
    if(nEvent == CV_EVENT_LBUTTONDOWN)
    {
    	selectedNum = 2;
    	outputfull = true;
    	setfull = false;
    }
}

void onMouse3(int nEvent, int x, int y, int nFlags, void *param)
{
    if(nEvent == CV_EVENT_LBUTTONDOWN)
    {
    	selectedNum = 3;
    	outputfull = true;
    	setfull = false;
    }
}

void onMouse4(int nEvent, int x, int y, int nFlags, void *param)
{
    if(nEvent == CV_EVENT_LBUTTONDOWN)
    {
    	selectedNum = 4;
    	outputfull = true;
    	setfull = false;
    }
}

void onMouse0(int nEvent, int x, int y, int nFlags, void *param)
{
    if(nEvent == CV_EVENT_LBUTTONDOWN)
    {
    	selectedNum = 0;
    	outputfull = false;
    	setfull = false;
    }
}

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
CameraImageViewer::CameraImageViewer(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inCam1In("inCam1", m_inCam1),
    m_inCam2In("inCam2", m_inCam2),
    m_inCam3In("inCam3", m_inCam3),
    m_inCam4In("inCam4", m_inCam4),
    m_selectOut("select", m_select)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
CameraImageViewer::~CameraImageViewer()
{
}



RTC::ReturnCode_t CameraImageViewer::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("inCam1", m_inCam1In);
  addInPort("inCam2", m_inCam2In);
  addInPort("inCam3", m_inCam3In);
  addInPort("inCam4", m_inCam4In);

  // Set OutPort buffer
  addOutPort("select", m_selectOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("image_height", m_image_height, "240");
  bindParameter("image_width", m_image_width, "320");
  // </rtc-template>
  selectedNum = 1;
  outputfull = false;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t CameraImageViewer::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageViewer::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageViewer::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CameraImageViewer::onActivated(RTC::UniqueId ec_id)
{
    //m_orig_img  = NULL;
    //small_image = NULL;
    //scale_image = NULL;

  //画像表示用ウィンドウの作成
    cvNamedWindow("Gorone1", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("Gorone2", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("Gorone3", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("Gorone4", CV_WINDOW_AUTOSIZE);

    //cvNamedWindow("Gorone", CV_WINDOW_AUTOSIZE);

    cvMoveWindow("Gorone4", 0 + m_image_width + X_MARGIN, Y_MARGIN + m_image_height + Y_MARGIN);
    cvMoveWindow("Gorone3", 0                           , Y_MARGIN + m_image_height + Y_MARGIN);
    cvMoveWindow("Gorone2", 0 + m_image_width + X_MARGIN, Y_MARGIN);
    cvMoveWindow("Gorone1", 0                           , Y_MARGIN);

    cvSetMouseCallback("Gorone1", onMouse1, (void*)this);
    cvSetMouseCallback("Gorone2", onMouse2, (void*)this);
    cvSetMouseCallback("Gorone3", onMouse3, (void*)this);
    cvSetMouseCallback("Gorone4", onMouse4, (void*)this);
    //cvSetMouseCallback("Gorone", onMouse4, (void*)this);
  return RTC::RTC_OK;
}


RTC::ReturnCode_t CameraImageViewer::onDeactivated(RTC::UniqueId ec_id)
{
    //表示ウィンドウの消去
    cvDestroyWindow("Gorone1");
    cvDestroyWindow("Gorone2");
    cvDestroyWindow("Gorone3");
    cvDestroyWindow("Gorone4");

    cvDestroyWindow("Gorone");
  return RTC::RTC_OK;
}


RTC::ReturnCode_t CameraImageViewer::onExecute(RTC::UniqueId ec_id)
{
    static coil::TimeValue tm_pre;
    static int count = 0;

    if (outputfull)
    {
      if (!setfull)
      {
        cvNamedWindow("Gorone", CV_WINDOW_AUTOSIZE);
        cvSetMouseCallback("Gorone", onMouse0, (void *)this);
        cvMoveWindow("Gorone", 0 + (m_image_width + X_MARGIN) * 2, Y_MARGIN);

        setfull = true;
      }
      switch (selectedNum)
      {
      case 1:
        if (m_inCam1In.isNew())
        {
          m_inCam1In.read();
          std::string str = "Gorone";
          if (ImageProgress(m_inCam1, str) < 0)
            return RTC::RTC_OK;
        }
        break;
      case 2:
        if (m_inCam2In.isNew())
        {
          m_inCam2In.read();
          std::string str = "Gorone";
          if (ImageProgress(m_inCam2, str) < 0)
            return RTC::RTC_OK;
        }
        break;
      case 3:
        if (m_inCam3In.isNew())
        {
          m_inCam3In.read();
          std::string str = "Gorone";
          if (ImageProgress(m_inCam3, str) < 0)
            return RTC::RTC_OK;
        }
        break;
      case 4:
        if (m_inCam4In.isNew())
        {
          m_inCam4In.read();
          std::string str = "Gorone";
          if (ImageProgress(m_inCam4, str) < 0)
            return RTC::RTC_OK;
        }
        break;
      default:
        break;
      }
    }
    else
    {
      cvDestroyWindow("Gorone");
      // Check input image is new
      if (m_inCam1In.isNew())
      {
        m_inCam1In.read();
        std::string str = "Gorone1";
        if (ImageProgress(m_inCam1, str) < 0)
          return RTC::RTC_OK;
      }

      if (m_inCam2In.isNew())
      {
        m_inCam2In.read();
        std::string str = "Gorone2";
        if (ImageProgress(m_inCam2, str) < 0)
          return RTC::RTC_OK;
      }

      if (m_inCam3In.isNew())
      {
        m_inCam3In.read();
        std::string str = "Gorone3";
        if (ImageProgress(m_inCam3, str) < 0)
          return RTC::RTC_OK;
      }

      if (m_inCam4In.isNew())
      {
        m_inCam4In.read();
        std::string str = "Gorone4";
        if (ImageProgress(m_inCam4, str) < 0)
          return RTC::RTC_OK;
      }
    }

    if (selectedNum > 0)
    {
      m_select.data = selectedNum;
      m_selectOut.write();
    }
    if (count > 100)
    {
        count = 0;
        coil::TimeValue tm;
        tm = coil::gettimeofday();
        double sec(tm - tm_pre);

        if (sec > 1.0 && sec < 1000.0)
        {
            std::cout << 100.0/sec << " [FPS]" << std::endl;
        }
        tm_pre = tm;
    }

    ++count;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t CameraImageViewer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageViewer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageViewer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageViewer::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageViewer::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

int CameraImageViewer::ImageProgress(RTC::CameraImage& m_in, std::string strGoron)
{
    if (!(m_in.pixels.length() > 0))
    {
        std::cout << "[ImageProgress] Size is 0" << std::endl;
        return -1;
    }

    cv::Mat orig_img = GetMatImage(m_in);
    cv::Mat small_img = cv::Mat::zeros(cv::Size(m_image_width, m_image_height), CV_8UC3);

    if(outputfull)
    {
    	    //画像表示
    	    cv::imshow(strGoron.c_str() , orig_img);
      	  cv::waitKey(2);
    }
    else
    {
	    //変換
        cv::resize(orig_img, small_img, small_img.size(), 0, 0, cv::INTER_LINEAR);
   	    //画像表示
  	    cv::imshow(strGoron.c_str() , small_img);
  	    cv::waitKey(2);
    }
return 0;
}


extern "C"
{

  void CameraImageViewerInit(RTC::Manager* manager)
  {
    coil::Properties profile(cameraimageviewer_spec);
    manager->registerFactory(profile,
                             RTC::Create<CameraImageViewer>,
                             RTC::Delete<CameraImageViewer>);
  }

};


